#ifndef MYPAINTCONFIG_H
#define MYPAINTCONFIG_H

#ifndef MYPAINT_TILE_SIZE
#define MYPAINT_TILE_SIZE 64
#endif

#ifndef MYPAINT_MAX_THREADS
#define MYPAINT_MAX_THREADS 16
#endif

#ifndef MYPAINT_MAX_MIPMAP_LEVEL
#define MYPAINT_MAX_MIPMAP_LEVEL 4
#endif

#endif /* MYPAINTCONFIG_H */
